var nls    = require('nls');
var _      = require('underscore');

const _private = {
    /**
     * This is a regular personummer validator, check length, date and luhnAlg
     * @param id
     * @returns {boolean}
     */
    isValidPersonNumber: function(id) {
        if(!id) return false;
        id = ("" + id);
        // accept only digits and length of 10
        if (!/^[0-9]{10}$/.test(id)) return false;

        //Check if first 4 numbers are a date. 650401-2233 => 65-04-01
        const isValidDate = new Date(id.substr(0,2) +
         "-" + id.substr(2,2)+
         "-" + id.substr(4,2)
        );

        // Not valid date object, return false
        if(!isValidDate) return false;

        // The Luhn Algorithm.
        var nCheck = 0, nDigit = 0, bEven = false;

        for (var n = id.length - 1; n >= 0; n--) {
            var cDigit = id.charAt(n),
             nDigit = parseInt(cDigit, 10);

            if (bEven) {
                if ((nDigit *= 2) > 9) nDigit -= 9;
            }

            nCheck += nDigit;
            bEven = !bEven;
        }

        return (nCheck % 10) == 0;
    }
};


/**
 * Contains helper functions to use for company data.
 *
 * @version 1.2.0
 *    0.0.1 - Vanilla
 *    1.0.0 - Initial
 *    1.1.0 - Added 'getScale' function
 *    1.2.0 - Changed code with es6 syntax to es5
 * @since 2.0.0
 */
var CompanyHelper = {
    getCompanyType: function(val, orgNr) {
        var data = this.getAllCompanyTypes();
        var value = data[val] || null;
        if (value === null && orgNr && this.isFilial(orgNr)) {
            value =  data.FIL;
        }
        return value;
    },
    getAllCompanyTypes: function() {
        return {
            ab: nls.companyTypeAB,
            ef: nls.companyTypeEF,
            fi: nls.companyTypeFI,
            sk: nls.companyTypeSK,
            hk: nls.companyTypeHK,
            fil: nls.companyTypeFIL,
        };
    },
    /**
     *  Validates custom foreign OrgNr
     * @param id
     * @returns {boolean}
     */
    isValidForeignOrgNr: function (id){
        if(!id) return false;
        id = id.toString();
        // 11 in length and prepended with '1000'
        return id.length === 11 && id.substr(0, 4) === '1000'
    },
    /**
     *  Checks if a number is a valid orgNr.
     *  Uses Luhn-alghoritm for validation. See README for more info
     *
     *  @param id `number` - The id to validate
     *  @version 1.0.0 - This is a final method, no changes should be necessary.
     *  @since 0.0.1
     */
    isValidOrgNr: function(id) {
        id = +id; //Cast to int
        if (_.isNumber(id) === false || _.isNaN(id) === true) {
            return false;
        }

        id = '' + id; //Cast to string

        // Check if it's an foreign company.
        if(id.length === 11 && id.substr(0, 4) === '1000') return true;

        if (id.length !== 10) {
            return false;
        }

        // Extra check for PersonNumbers, luhn alg below has been proven to be inconsistant
        if(_private.isValidPersonNumber(id) === true) return true;

        var total = 0;
        for (var i = 0, j = 1, len = id.length; i < len; i += 1, j += 1) {
            var v = +id[i];
            if (i === 2 && v < 2) {
                return false;
            }

            v = v * ((j % 2) + 1);
            if (+v > 9) {
                var a = '' + v;
                total += +(+a[0] + +a[1]);
            } else {
                total += +v;
            }
        }

        if (total % 10 == 0) {
            return true;
        } else {
            return false;
        }
    },
    /**
     *  Checks if the orgNr is a filialnumber or not
     *
     *  @param id `number` - The id to validate
     *  @version 1.0.0 - FINAL
     *  @since 1.0.0
     */
    isFilial: function(id) {
        if(!this.isValidOrgNr(id)) {
            return false;
        }
        return /^699.{7}$/.test(id);
    },
    /**
     * Returns standard scales for a given company property.
     * If no key is passed, returns scales for all properties with scales.
     *
     * @author Irfan Agovic
     * @name _getStandardScales
     * @access private
     * @version 1.0.0 - Initial
     * @since 1.1.0
     * @param {string} [key] Property to get scales for. Optional, valid values: ['antanst_AB_X', 'netoms_AB_X']
     */
    _getStandardScales: function(key) {
        var scales = {
            antanst_AB_X: {
                0: { from: 0, to: 0 },
                1: { from: 1, to: 9, },
                2: { from: 10, to: 49, },
                3: { from: 50, to: 99, },
                4: { from: 100, to: 199, },
                5: { from: 200, to: 499, },
                6: { from: 500, to: null, },        // More than 499
            },
            netoms_AB_X: {
                1: { from: null, to: 0, },          // Less than 1
                2: { from: 1, to: 499, },
                3: { from: 500, to: 999, },
                4: { from: 1000, to: 4999, },
                5: { from: 10000, to: 49999, },
                6: { from: 50000, to: 499999, },
                7: { from: 500000, to: null, },     // More than 499999
            },
        };

        if (key && !scales[key]) {
            // Given prop doesnt have any scales, throw error.
            throw new Error('Invalid prop "' + prop + '" given to _getStandardScales() in CompanyHelper');
        }

        if (key) {
            // Prop-specific standard scales
            return scales[key];
        } else {
            // All props' standard scales
            return scales;
        }
    },
    /**
     * Returns a scale for a given company property, out of the provided keys.
     * The keys are used to get scales from the properties' standard scales list.
     *
     * @author Irfan Agovic
     * @name getScale
     * @access public
     * @version 1.0.0 - Final.
     * @since 1.1.0
     */
    getScale: function(prop, keys) {
        if (!(keys && keys.length)) {
            // No given keys, throw.
            throw new Error('Invalid keys given to getScale() in CompanyHelper');
        }

        // Typecast to numbers
        keys = _(keys).map(function(val) { return _.isObject(val) ? +(val.val) : +val; });

        // Standard scales for the property
        var standardScales = this._getStandardScales(prop);
        var getScaleByKey = function(key) { return standardScales[key]; };
        var scales = _(keys).map(getScaleByKey);

        if (scales.length === 1) {
            // If theres only one scale, return it.
            return scales[0];
        } else {
            // Otherwise create a new one from lowest to highest.
            var lowest = _(scales).min(function(scale) { return scale.to; });
            var highest = _(scales).max(function(scale) { return scale.from; });

            return { from: lowest.from, to: highest.to, };
        }
    },
};

module.exports = CompanyHelper;
