var _ = require('underscore');

/**
 * Helper functions for handling settings.
 *
 * @author Irfan Agovic
 * @name SettingsHelper
 * @access public
 * @since 2.0.0
 * @version 1.1.0
 *   1.0.0 Initial release.
 *   1.0.1 Refactored to using es6 syntax and styling (airbnb).
 *   1.1.0 Changed code from es6 syntax to es5 (ironically)
 */
var SettingsHelper = {
    settings: {
        longNew:         1,
        shortNew:        2,
        analyzeDay:      4,
        analyzeMonth:    8,
        eyeDay:          16,
        eyeMonth:        32,
        email:           64,
        privateProspect: 128,
        companyProspect: 256,
        carProspect:     512,
    },
    categories: {
        prospect: ['privateProspect', 'companyProspect', 'carProspect'],
        new:      ['longNew', 'shortNew'],
        analyze:  ['analyzeDay', 'analyzeMonth'],
        eye:      ['eyeDay', 'eyeMonth'],
        email:    ['email'],
    },
    /**
     * Finds the setting bit for a setting flag.
     *
     * @author Irfan Agovic
     * @function
     * @name getSettingBitFromFlag
     * @access public
     * @version 1.0.0
     * @since 1.0.0
     * @param {string} flag Setting to check for.
     * @returns {number} Setting bit if the flag is valid, otherwise 0 (false).
     */
    getSettingBitFromFlag: function(flag) {
        if (this.settings[flag] !== undefined) {
            return this.settings[flag];
        }
        return 0;
    },
    /**
     * Determines if the user has saved a specific setting.
     *
     * @author Irfan Agovic
     * @function
     * @name hasSavedSetting
     * @access public
     * @version 1.0.0
     * @since 1.0.0
     * @param {string} flag Setting to check for.
     * @param {number} userSettings User's setting bit.
     * @returns {boolean} Whether the setting is saved or not.
     */
    hasSavedSetting: function(flag, userSettings) {
        return ((userSettings & this.getSettingBitFromFlag(flag)) !== 0);
    },
    /**
     * Finds all setting flags that are saved for a specific setting bit.
     * Returns null if no flags are found.
     *
     * @author Irfan Agovic
     * @function
     * @name getSavedSettings
     * @version 1.0.0
     * @since 1.0.0
     * @function getSavedSettings
     * @param {number} settingBit Setting bit to look in.
     * @returns {?Array<string>} The found setting flags or null if none were found.
     */
    getSavedSettings: function(settingBit) {
        var saved = [];
        var _this = this;

        /* Loops through all setting flags to find the ones that are stored in the setting bit */
        _(this.settings).each(function(bit, flag) {
            if (_this.hasSavedSetting(flag, settingBit)) {
                // Found a saved setting, push to buffer
                saved.push(flag);
            }
        });

        return (_.isEmpty(saved) === false) ? saved : null;
    },
    /**
     * Categorizes the given setting flags.
     * Returns null if no valid flags are given.
     *
     * @author Irfan Agovic
     * @function
     * @name categorizeFlags
     * @access public
     * @version 1.0.1 Refactored code into smaller functions.
     *     1.0.0 Initial release.
     * @since 1.0.0
     * @param {Array<string>} settings Array of setting flags.
     * @returns {?Object} An object of categorized flags or null if flags are invalid.
     */
    categorizeFlags: function(settings) {
        var categorized = {};
        var _this = this;

        var categorizeFlag = function(flag, category) {
            var cat = _this.categories[category];
            if (_.contains(cat, flag)) {
                categorized[category] = flag;
            }
        };

        var categorize = function(flag) {
            var categories = [ 'prospect', 'new', 'analyze', 'eye', ];
            _(categories).each(function(category) {
                categorizeFlag(flag, category);
            });
        };

        // Categorize!
        _(settings).each(categorize);

        // Email is a special case - it can only be true/false
        categorized['email'] = _(settings).contains('email') ? true : false;

        return _.isEmpty(categorized) === false ? categorized : null;
    },
    /**
     * Computes a setting bit from a list of setting flags.
     * Takes either an array of flags as argument, or an object of categorized settings together with an optional argument.
     *
     * @author Irfan Agovic
     * @since 1.0.0
     * @version 1.0.1 Re-styled according to airbnb guideline.
     *     1.0.0 Initial release.
     * @public
     * @function computeSettingBit
     * @param {(Array<string>|Object)} flags Setting flags to calculate from.
     * @param {boolean} [categorized] (optional) Has to be passed if function should handle a categorized object, and has to be true.
     * @returns {?number} The resulting setting bit or null if the given arguments (including values/properties) are invalid.
     */
    computeSettingBit: function(flags, categorized) {
        var settingBit = 0;
        var shouldLoop = false;
        var flagsClone = _.clone(flags);
        var _this = this;
        
        if (categorized === true) {
            // Categorized object is given, validate its properties
            var objectCategories = _.keys(flagsClone);
            var validCategories  = _.keys(_this.categories);
            var diff = _(objectCategories).difference(validCategories);
            var flagsAreValid = _.isEmpty(diff) === true;

            if (flagsAreValid) {
                // Valid properties. Turn object into array of setting flags and say it's OK to loop through it.
                // Email is a special case, can only have a boolean value.

                if (flagsClone.email === false) {
                     // No email, remove flag
                    flagsClone = _(flagsClone).omit('email');    
                } else {
                    // Otherwise give it the email setting flag as value
                    flagsClone['email'] = 'email';
                }
                flagsClone = _.values(flagsClone);
                shouldLoop = true;
            }
        } else {
            // Otherwise validate that the passed argument is an array
            shouldLoop = _.isArray(flagsClone);
        }

        if (shouldLoop === true) {
            var validFlags = _.keys(this.settings);
            _(flagsClone).each(function(flag) {
                if (_(validFlags).contains(flag) === false) {
                    // There's an invalid flag - don't return a setting bit
                    shouldLoop = false;
                }
                settingBit += _this.getSettingBitFromFlag(flag);
            });
        }
        return (shouldLoop === true) ? settingBit : null;
    }
};


module.exports = Object.create(SettingsHelper);

